/**********************************************************************************************
 * 
 * ADOBE CONFIDENTIAL
 * 
 * Copyright 2014 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE: All information contained herein is, and remains the property of Adobe Systems
 * Incorporated and its suppliers, if any. The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its suppliers and are protected
 * by trade secret or copyright law. Dissemination of this information or reproduction of
 * this material is strictly forbidden unless prior written permission is obtained from
 * Adobe Systems Incorporated.
 * 
 **********************************************************************************************/

 (function() {


// Adobe Event Bridge Explained:
// 
// A gesture on a web view should sometimes be consumed by the HTML/JS content, and should 
// sometimes be consumed by the viewer. There are clear cases (such as a single tap on an anchor), 
// where it should always be consumed by the HTML/JS content. In other cases, it depends. A swipe 
// over HTML content that contains an embedded slideshow should be consumed by the HTML/JS content. 
// However, a swipe over content that contains plain text should be consumed by the viewer. 
// 
// The goal of this file is to help differentiate between those events that are consumed by the 
// HTML/JS content and those events that should be consumed by the folio viewer. Since JavaScript 
// doesn't have a way to tell whether an event has been consumed, we make our best guess. 
// 
// The guessing is based on a combination of event target and event listeners, but it is not 
// perfect. For example, it’s a good guess that a tap on an anchor tag will be consumed by the 
// HTML content. However, it’s less clear whether a canvas with a mousedown event listener 
// would consume the event. 
// 
// There are two complementary approaches to obtaining the event target and event listener:
// 		1) Track calls to addEventListener. For example, if a click listener has been added to an 
// element, it's a good guess that click events on that element will be consumed by the 
// HTML/JS content. 
// 		2) Walk the DOM and look at element attributes. Note that modern single page web applications
// can continually modify the DOM, so DOM changes are also evaluated.
//
// Approach #1 is done in the adobe_event_bridge_early.js file, because it requires that our tracking
// is in place before the publisher's content is rendered. Approach #2 is done in this file, which 
// can be applied after the publisher's content is rendered.
//

//
// All Android versions have their share of webview bugs, but API 15 has some very
// unique ones. The Java layer will tell us if we're operating in the context of 
// API 15 so we can apply the proper workarounds.
var isApi15 = VALUE_FROM_JAVA__IS_API_15;


	// Certain content like only sends touch events, so we work with touch events instead of mouse.
	listenForEvent("touchstart");
	listenForEvent("touchmove");
	listenForEvent("touchend");
	listenForEvent("touchcancel");
	listenForEvent("touchleave");


	// Listen for events that the viewer cares about. 
	function listenForEvent(eventName) {
		if( this[eventName+"_listener"] != null )
			return;

		var handler = this[eventName+"_listener"] = function(event){handleEventBubblePhase(event)};

		// Note the addition of the last parameter (custom to Adobe DPS): true to apply event listener
		// directly to HTML (and not apply Adobe event suppression).
		window.addEventListener(eventName, handler, false, true);
	};


	// orig is the touchdown location of the first event in gesture.
	var origX = 0;
	var origY = 0;

	// dX/dY are the total change since the beginning of the gesture.
	var dX = 0 ;
	var dY = 0 ;

	//
	// Returns a boolean whether the HTML can scroll based on the given event.
	// 
	// This is a first pass, with the second pass happening in the Java layer. 
	// There are certain scrolling situations that are only detectable from here
	// inside the HTML/JS, and there are other scrolling situations that are only.
	// detectable from outside in the Java layer, so a two-pass system is necessary
	// to respond correctly to all of them.
	// 
	// A true value holds, while a false value can be overwritten in the second pass.
	// In other words, if this method returns true, then the scrolling will absolutely 
	// be handled by the HTML/JS layer. However, if this method returns false, there 
	// is still a chance (based on the results from the Java layer) that the HTML/JS 
	// will handle the scroll.
	//
	function canScroll(event) {
		var canscroll = false;

		if (event.changedTouches && event.changedTouches[0]) {
			 dX = event.changedTouches[0].pageX - origX;
			 dY = event.changedTouches[0].pageY - origY;
		}	

		var slope = Math.abs(dY / dX);

		//	slope < 1 is mostly horizontal
		if (slope < 1) {

			var widthDifferential = 0;

			// hack for API 15: width dimensions can be off by 1. not sure why.
			if (isApi15) {
				widthDifferential = 1;
			}

			// dX > 0 so finger swiped right.
			if (dX > 0 && window.scrollX > 0) {

				// we don't have the data to figure this one out, but the java layer does, so we
				// return false to send it there.
				//
				// the problem is that a positive scrollX can either mean the content is scrolled
				// (what you'd expect) or it can mean the content is partially offscreen. the 
				// java layer can disambiguate these cases, but the HTML/JS layer cannot.

				// workaround for Android bugs: 
				// https://code.google.com/p/android/issues/detail?id=4549 & 
				// https://code.google.com/p/android/issues/detail?id=19827
				// call into Java because API 15 needs to know before it decides to preventDefault().
				if (isApi15) {
					if (_adobedps_native_overlay.api15CanScroll(window.scrollX, window.scrollY, dX, dY)) {
						canscroll = true;
					}
				}
			} 
			// dX < 0 so finger swiped left
			else if (dX < 0 && (window.document.documentElement.scrollWidth - (window.scrollX + innerWidth)) > widthDifferential) {
				canscroll = true;
			}
			// else do nothing and return false. the java layer can override false.
		}
		// slope > 1 is mostly vertical
		else if (slope >= 1) {
			// dY > 0 so finger swiped down.
			if (dY > 0 && window.scrollY > 0 ) {

				// we don't have the data to figure this one out, but the java layer does, so we
				// return false to send it there.
				//
				// the problem is that a positive scrollY can either mean the content is scrolled
				// (what you'd expect) or it can mean the content is partially offscreen. the 
				// java layer can disambiguate these cases, but the HTML/JS layer cannot.

				// workaround for Android bugs: 
				// https://code.google.com/p/android/issues/detail?id=4549 & 
				// https://code.google.com/p/android/issues/detail?id=19827
				// call into Java because API 15 needs to know before it decides to preventDefault().
				if (isApi15) {
					if (_adobedps_native_overlay.api15CanScroll(window.scrollX, window.scrollY, dX, dY)) {
						canscroll = true;
					}
				}
			} 
			// dY < 0 so finger swiped up
			else if (dY < 0 && (window.document.documentElement.scrollHeight - (window.scrollY + window.innerHeight)) > 0) {
				canscroll = true;
			}
			// else do nothing and return false. the java layer can override false.
			
		}

		return canscroll;
	};

	var gestureIgnored = false;

	// This is our own event listener, which will send events up to the viewer 
	// with a boolean stating whether they've been consumed.
	// 
	// This handling needs to remain FAST! If it takes longer than 200ms to respond to a 
	// touchstart event, Android will send us a touchcancel:
	// https://code.google.com/p/chromium/issues/detail?id=260732
	// 
	// The above reason is why we walk the DOM elements in onPageFinished instead of computing on-the-fly.
	// 
	function handleEventBubblePhase(event) {

		var eventObj = {event:{type:event.type}};

		if (event.touches && event.touches.length > 1 ) {
			gestureIgnored = true;			
		}

		if (!gestureIgnored) {

			if (event.type === "touchstart") {
				if (event.defaultPrevented || (event.ADOBEDMPNS && event.ADOBEDMPNS.suppressEventFromViewer) ) {

					// CONTENT DECISION: Should presence of listener on touchstart events kill entire gesture? 
					// For some content this makes sense; for other content it doesn't. for now, we say no.
					// gestureIgnored = true;
				} else {
					if (event.changedTouches && event.changedTouches[0]) {
						origX = event.changedTouches[0].pageX;
						origY = event.changedTouches[0].pageY;
					}
				}
			} else if (!event.defaultPrevented && !gestureIgnored && 
					!(event.ADOBEDMPNS && event.ADOBEDMPNS.suppressEventFromViewer)) {

				if (event.type === "touchmove") {

					if (canScroll(event)) {
						eventObj.consumed = true;
					} else if (isApi15) {
						// workaround for Android bugs: 
						// https://code.google.com/p/android/issues/detail?id=4549 & 
						// https://code.google.com/p/android/issues/detail?id=19827
						event.preventDefault();
					} 

				} else if (event.type === "touchend" || event.type === "touchleave" || event.type === "touchcancel") {
				}

				eventObj.sent = true;
				send( eventObj );

			} else {
				gestureIgnored = true;
			}
		}

		if (event.type === "touchend" || event.type === "touchleave" || event.type === "touchcancel"  || 
			// API 15 hacks: we rely on a roundtrip to know if we can communicate with webview.
			(isApi15 && (event.type === "touchstart"))) {

				if (!eventObj.sent){ 
					eventObj.consumed = gestureIgnored;

					send(eventObj);
				}

				gestureIgnored = false;
		}

	};

	// send the event to the viewer.
	function send(data) {
		_adobedps_native_overlay.jsEvent( data.event.type, window.scrollX, window.scrollY, dX, dY, Boolean(data.consumed) );
	};
		
})();