/**********************************************************************************************
 * 
 * ADOBE CONFIDENTIAL
 * 
 * Copyright 2014 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE: All information contained herein is, and remains the property of Adobe Systems
 * Incorporated and its suppliers, if any. The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its suppliers and are protected
 * by trade secret or copyright law. Dissemination of this information or reproduction of
 * this material is strictly forbidden unless prior written permission is obtained from
 * Adobe Systems Incorporated.
 * 
 **********************************************************************************************/

 (function() {

// The Adobe Event Bridge provides the native Adobe Viewer code the opportunity to communicate 
// with the JavaScript code about events. For a more detailed explanation of the Adobe Event 
// Bridge, please see adobe_event_bridge.js. 

// This file contains the bare minimum items that absolutely have to be done early in web page
// loading. In general, we should NOT be attempting to inject javascript early, as the DOM is 
// not yet complete (so we can't attempt tasks like walking the DOM, for example). Unfortunately, 
// page finish is too late for intercepting the attachment of event listeners to the document.

	// sets up event suppression when future event listeners are added.
	setUpEventHandlingMagic();

	// 
	// Event Handling Magic Explained:
	// 
	// Here is where calls to addEventListener are tracked by replacing the function wtih a 
	// custom version that inserts trackability.
	// 
	// Note that there are three separate places to add an event listener (window, document, 
	// element), so there are three parts to this setup.
	// 
	// While we'd like to just be able to evaluate the events real-time, we cannot trust the 
	// HTML content to properly denote whether it has handled the event. Even respected sites 
	// like YouTube, for example, do not call preventDefault() on events, so we have no way of 
	// knowing whether an event has already been handled.
	function setUpEventHandlingMagic() { 

		// 
		// DUPLICATE CODE WARNING: This method also appears in adobedps_walk_dom_elements.js
		// 
		// called at event handling time when we guess an event is being consumed by HTML/JS.
		function suppressEventFromViewer(event) {


			if (!event.ADOBEDMPNS) {
				event.ADOBEDMPNS = {};
			}

			// We set this flag here, and then the JavaScript
			// bridge looks for it before forwarding events 
			// to the Viewer.
			//
			// jQuery UI adds event listener to the HTMLDocument, this would suppress event 
			// and prevent HUD from being shown(Bug#3770854). To workaround it, we chose to
			// not suppress the event sent to HTMLDocument when both jQuery and jQuery UI are 
			// loaded.
			if (!(window.jQuery && window.jQuery.ui) ||
				!(event.currentTarget instanceof HTMLDocument)) {
					event.ADOBEDMPNS.suppressEventFromViewer = true;
			}
		}


		// the presence of any of these event listeners on an object imply that an event targeted
		// toward that object will be consumed at the webkit layer.
		var magicEvents = {
			click: true,
			mousedown: true,
			mouseup: true,
			mousemove: true,
			touchstart: true,
			touchmove: true,
			touchend: true
		};

		// target-agnostic event tracking. used for tracking event listeners at the window, 
		// document, and folio level.
		function trackEventListener(target, type, listenerToAdd, originalListener, useCapture) {


			// Ensures DPS touch event listeners are properly impacted by the content's other
			// listeners (mouse, click, etc.).
			//
			// we use touchend for non-move events because that's the one that gets sent to the viewer.
			if (type == "mouseup") {
				originalListener.call(target, "touchend", suppressEventFromViewer, useCapture);
			}
			else if (type == "mousedown") {
				originalListener.call(target, "touchend", suppressEventFromViewer, useCapture);
			}
			else if (type == "mousemove") {
				originalListener.call(target, "touchmove", suppressEventFromViewer, useCapture);
			}			
			else if (type == "click") {
				originalListener.call(target, "touchend", suppressEventFromViewer, useCapture);
			}

			// have JS engine's addEventListener call our supression code.
			if (magicEvents[type]) {
				originalListener.call(target, type, suppressEventFromViewer, useCapture);
			}

			// have JS engine's addEventListener call the content's event listener.
			if (listenerToAdd !== suppressEventFromViewer) {
				originalListener.call(target, type, listenerToAdd, useCapture);
			}
		}

		//		
		// Part I: track calls to addEventListener on HTML elements.
		//
		// Note the addition of the last parameter (custom to Adobe DPS): true to apply event listener
		// directly to HTML (and not apply Adobe event suppression).
		//
		var defaultAddEventListener = HTMLElement.prototype.addEventListener;
		HTMLElement.prototype.addEventListener = function(type, listener, useCapture, direct) {

			// evaluated from the context of the target, so "this" refers to the target element
			if (!direct) {
				trackEventListener(this, type, listener, defaultAddEventListener, useCapture);
			} else {
				defaultAddEventListener.call(this, type, listener, useCapture);
			}
		};

		// Part II: track calls to addEventListener on HTML document.
		//
		// Note the addition of the last parameter (custom to Adobe DPS): true to apply event listener
		// directly to HTML (and not apply Adobe event suppression).
		//
		var defaultDocAddEventListener = HTMLDocument.prototype.addEventListener;
		HTMLDocument.prototype.addEventListener = function(type, listener, useCapture, direct) {

			// evaluated from the context of the target, so "this" refers to the target HTMLDocument
			if (!direct) {
				trackEventListener(this, type, listener, defaultDocAddEventListener, useCapture);
			} else {
				defaultDocAddEventListener.call(this, type, listener, useCapture);
			}
		};

		// Part III: track calls to addEventListener on window.
		//
		// Note the addition of the last parameter (custom to Adobe DPS): true to apply event listener
		// directly to HTML (and not apply Adobe event suppression).
		//
		var defaultWindowAddEventListener = window.addEventListener;	

		window.addEventListener = function(type, listener, useCapture, direct) {

			// evaluated from the context of the target, so "this" refers to the target window
			if (!direct) {
				trackEventListener(this, type, listener, defaultWindowAddEventListener, useCapture);
			} else {
				defaultWindowAddEventListener.call(this, type, listener, useCapture);
			}
		};

	};
})();