/**********************************************************************************************
 * 
 * ADOBE CONFIDENTIAL
 * 
 * Copyright 2014 Adobe Systems Incorporated. All Rights Reserved.
 * 
 * NOTICE: All information contained herein is, and remains the property of Adobe Systems
 * Incorporated and its suppliers, if any. The intellectual and technical concepts contained
 * herein are proprietary to Adobe Systems Incorporated and its suppliers and are protected
 * by trade secret or copyright law. Dissemination of this information or reproduction of
 * this material is strictly forbidden unless prior written permission is obtained from
 * Adobe Systems Incorporated.
 * 
 **********************************************************************************************/

 (function() {

	// sets up event suppression when future event listeners are added.
	setUpEventHandlingMagic();

	// 
	// Walk the DOM and look for element attributes that indicate events will be handled 
	// at the HTML/JS layer.
	// 
	function setUpEventHandlingMagic() { 

		// 
		// DUPLICATE CODE WARNING: This method also appears in adobe_event_bridge_early.js
		// 
		// called at event handling time when we guess an event is being consumed by HTML/JS.
		function suppressEventFromViewer(event) {

			if (!event.ADOBEDMPNS) {
				event.ADOBEDMPNS = {};
			}

			// We set this flag here, and then the JavaScript
			// bridge looks for it before forwarding events 
			// to the Viewer.
			event.ADOBEDMPNS.suppressEventFromViewer = true;
		};

		///////////////////////////////////////////////////
		// T A P  -  O N L Y
		///////////////////////////////////////////////////

		// the set of events that will be suppressed from the viewer for tap-consumers.
		// these events will be suppressed when any of the below elements/atributes
		// are encountered in the DOM.
		var tapEvents = {
			click: true,
			mouseup: true,
			touchend: true
		};
		// these objects will always be consumed at the HTML/JS layer.
		var tapElements = {
			textarea: true,
			input: true
		};		
		// the presence of any of these attributes on an object imply that an event targeted
		// toward that object will be consumed at the HTML/JS layer.
		var tapAttributes = {
			onmousedown: true,
			onmouseup: true,
			ondblclick: true,
			onclick: true
		};

		///////////////////////////////////////////////////
		// S W I P E  -  O N L Y
		///////////////////////////////////////////////////

		// 
		// the set of events that will be suppressed from the viewer for swipe-consumers.
		// these events will be suppressed when any of the below elements/atributes
		// are encountered in the DOM.
		var swipeEvents = {
			mousemove: true,
			touchmove: true,
		}
		// the presence of any of these attributes on an object imply that an event targeted
		// toward that object will be consumed at the HTML/JS layer.
		var swipeAttributes = {
			onmousemove: true,
			onscroll: true
		};

		///////////////////////////////////////////////////
		// S W I P E   &   T A P
		///////////////////////////////////////////////////

		// these objects will always be consumed at the HTML/JS layer.
		//
		// the superset of tap & swipe events will be suppressed.
		var tapAndSwipeElements = {
			video: true,
			audio: true
		};		

		
		// util: adds viewer-level event suppression for the parameter object. Viewer-level
		// event suppression means that the event will be reported to the viewer as 
		// already consumed, so the viewer should not respond to the event. Viewer-level
		// event suppression has no impact on how the HTML/JS content handles the event.
		function suppressEventsForObject(events, object) {

			for (var i in events) {

				// Note the addition of the last parameter (custom to Adobe DPS): true to apply event listener
				// directly to HTML (and not apply Adobe event suppression).
				object.addEventListener(i, suppressEventFromViewer, false, true);
			}
		};

		// util: whether an object has any of the set of attributes.
		function hasAttribute(object, attributes) {
			for (var attr in attributes) {
				if (object.getAttribute(attr)) {
					return true;
				}
			}
			return false;
		};
		
		// evaluates the parameter object for viewer-level event suppression. Viewer-level
		// event suppression means that the event will be reported to the viewer as 
		// already consumed, so the viewer should not respond to the event. Viewer-level
		// event suppression has no impact on how the HTML/JS content handles the event.
		function suppressDOMLevel2EventsOnMagicElements(object) {
			var tagName = object.nodeName.toLowerCase();
			var hasTapAttr = hasAttribute(object, tapAttributes);
			var hasSwipeAttr = hasAttribute(object, swipeAttributes);

			// for taps, our best guess resolves to:
			// 1) anchor elements w/href
			// 2) a set of non-anchor elements like "textarea" (full set above)
			// 3) elements containing attributes like "onclick" (full set above)
			if (
				(tagName === "a" && object.getAttribute("href")) ||
				tapElements[tagName] ||
				hasTapAttr
				) {
					suppressEventsForObject(tapEvents, object);
			}

			// for swipes, our best guess resolves to:
			// 1) elements containing attributes like "onmousemove" (full set above)
			if ( hasSwipeAttr ) {
					suppressEventsForObject(swipeEvents, object);
			}

			// some elements should consume both swipes & taps. 
			// our best guess of those resolves to:
			// 1) a set of elements like "video" (full set above)
			if ( tapAndSwipeElements[tagName] ) {
					suppressEventsForObject(tapEvents, object);
					suppressEventsForObject(swipeEvents, object);
			}

		};

		// DOM is dynamic - this handler takes care of newly inserted elements.
		function domNodeInsertedHandler(event) {
			if (event.target.nodeType == Node.ELEMENT_NODE) {
				suppressDOMLevel2EventsOnMagicElements(event.target);
			}
		};

		// sets up event suppression for future inserted DOM elements.
		//
		// TODO: evaluate the use of mutation observers instead, as they should be much less
		// expensive. See: https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
		window.document.addEventListener("DOMNodeInserted", domNodeInsertedHandler);

		// sets up event suppression for existing DOM elements.
		walkDOMElements(document.body, suppressDOMLevel2EventsOnMagicElements);	
			
	};

	// walk through the existing DOM elements and call the callback function. 
	function walkDOMElements(object, callback) {
		if (object && object.nodeType && object.nodeType === Node.ELEMENT_NODE) {
			callback(object);

			for (var kid = object.firstChild; kid; kid = kid.nextSibling) {
                walkDOMElements(kid, callback);
			}
		}
	};

})();
